﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Mvc.Ajax;
using MvcGuestBook.Models;
using System.Text.RegularExpressions;
using MvcGuestBook.Components;

namespace MvcGuestBook.Controllers
{
    [MyHandleError]
    public class HomeController : Controller
    {
        GuestBookDataContext db = new GuestBookDataContext();

        public ActionResult Index()
        {
            ViewData["Title"] = "Home Page";
            ViewData["Message"] = "Welcome to ASP.NET MVC!";

            return View();
        }

        public ActionResult GuestBookList(int? page)
        {
            //测试ajax载入效果
            //System.Threading.Thread.Sleep(5000);

            //读取全部的留言
            int count = db.GuestBook.Count();
            int pagesize = 5;
            int pagecount = count / pagesize + ((count % pagesize == 0) ? 0 : 1);
            int pageindex = 1;
            if (page > pagecount)
            {
                pageindex = pagecount;
            }
            else
            {
                if (page > 0)
                {
                    pageindex = page ?? 1;
                }
            }
            this.ViewData["RecordCount"] = count;
            this.ViewData["PageCount"] = pagecount;
            this.ViewData["PageIndex"] = pageindex;

            var gs = db.GuestBook.OrderByDescending(g => g.CreatedDate).Skip((pageindex - 1) * pagesize).Take(pagesize);
            return View(gs.ToList());
        }

        public ActionResult About()
        {
            ViewData["Title"] = "About Page";

            return View();
        }

        [ActionName("GuestBookAdd"), AcceptVerbs(HttpVerbs.Post)]
        public ActionResult GuestBookSave(GuestBook objGuestBook)
        {
            try
            {
                bool hasError = false;
                if (objGuestBook.CreatedByUser == string.Empty)
                {
                    hasError = true;
                    this.ViewData.ModelState.AddModelError("CreatedByUser", "昵称不能为空！");
                }
                if (!IsValidEmail(objGuestBook.Email))
                {
                    hasError = true;
                    this.ViewData.ModelState.AddModelError("Email", "Email格式不正确！");
                }
                if (objGuestBook.Body == string.Empty)
                {
                    hasError = true;
                    this.ViewData.ModelState.AddModelError("Body", "留言内容不能为空！");
                }

                if (hasError)
                {
                    throw new InvalidOperationException();
                }

                //string str = objGuestBoook.Body;
                if (objGuestBook.ItemID == 0)
                {
                    db.GuestBook.InsertOnSubmit(objGuestBook);
                }
                else
                {
                    var gs = from g in db.GuestBook where g.ItemID == objGuestBook.ItemID select g;
                    foreach (GuestBook g in gs)
                    {
                        g.Body = objGuestBook.Body;
                        g.Email = objGuestBook.Email;
                        g.CreatedByUser = objGuestBook.CreatedByUser;
                    }
                }
                db.SubmitChanges();
            }
            catch
            {
                ViewData["Title"] = "添加新留言";
                ViewData["Message"] = "添加新留言";
                
                return View(objGuestBook);
            }
            return RedirectToAction("Index");
        }
        
        //[Authorize(Roles = "Administrator")]
        public ActionResult GuestBookDelete(int ItemID)
        {
            var gs = from g in db.GuestBook where g.ItemID == ItemID select g;
            foreach (GuestBook g in gs)
            {
                db.GuestBook.DeleteOnSubmit(g);
            }
            db.SubmitChanges();
            return RedirectToAction("Index", "Home");
        }

        [AcceptVerbs(HttpVerbs.Get)]
        public ActionResult GuestBookAdd(int? ItemID)
        {
            GuestBook objG = new GuestBook();

            string username = this.User.Identity.Name.Trim();
            if (username != string.Empty)
            {
                var us = from u in db.User where u.Username == username select u;
                User objU = null;
                foreach (User u in us)
                {
                    objU = u;
                }
                if (objU != null)
                {
                    objG.CreatedByUser = objU.Username;
                    objG.Email = objU.Email;
                    objG.CreatedByUserId = objU.UserID;
                }
            }

            ItemID = ItemID ?? 0;
            if (ItemID == 0)
            {
                ViewData["Title"] = "添加新留言";
                ViewData["Message"] = "添加新留言";
            }
            else
            {
                ViewData["Title"] = "编辑留言";
                ViewData["Message"] = "编辑留言";

                //读取全部的留言
                var gs = from g in db.GuestBook where g.ItemID == ItemID select g;
                foreach(GuestBook g in gs)
                {
                    objG = g;
                }
            }

            return View(objG);
        }

        private bool IsValidEmail(string strIn)
        {
            return Regex.IsMatch(strIn, @"^([\w-\.]+)@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.)|(([\w-]+\.)+))([a-zA-Z]{2,4}|[0-9]{1,3})(\]?)$");
        }
    }
}
